@@ -39,7 +39,7 @@ struct connman_profile {
 	connman_bool_t offlinemode;
 };
 
-static GHashTable *profiles = NULL;
+static GHashTable *profile_hash = NULL;
 static struct connman_profile *default_profile = NULL;
 
 static DBusConnection *connection = NULL;
@@ -53,99 +53,30 @@ static void append_path(gpointer key, gpointer value, gpointer user_data)
 							&profile->path);
 }
 
-void __connman_profile_list(DBusMessageIter *iter)
+void __connman_profile_list(DBusMessageIter *iter, void *user_data)
 {
-	DBG("");
-
-	g_hash_table_foreach(profiles, append_path, iter);
-}
-
-static void append_profiles(DBusMessageIter *entry)
-{
-	DBusMessageIter value, iter;
-	const char *key = "Profiles";
-
-	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
-		DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_OBJECT_PATH_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-	__connman_profile_list(&iter);
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(entry, &value);
+	g_hash_table_foreach(profile_hash, append_path, iter);
 }
 
 static void profiles_changed(void)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry;
-
-	DBG("");
-
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-	append_profiles(&entry);
-	g_dbus_send_message(connection, signal);
+	connman_dbus_property_changed_array(CONNMAN_MANAGER_PATH,
+			CONNMAN_MANAGER_INTERFACE, "Profiles",
+			DBUS_TYPE_OBJECT_PATH, __connman_profile_list, NULL);
 }
 
 static void name_changed(struct connman_profile *profile)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "Name";
-
-	DBG("profile %p", profile);
-
-	signal = dbus_message_new_signal(profile->path,
-				CONNMAN_PROFILE_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_STRING_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_STRING,
-							&profile->name);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
+	connman_dbus_property_changed_basic(profile->path,
+				CONNMAN_PROFILE_INTERFACE, "Name",
+					DBUS_TYPE_STRING, &profile->name);
 }
 
 static void offlinemode_changed(struct connman_profile *profile)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "OfflineMode";
-
-	DBG("profile %p", profile);
-
-	signal = dbus_message_new_signal(profile->path,
-				CONNMAN_PROFILE_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN,
-						&profile->offlinemode);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
+	connman_dbus_property_changed_basic(profile->path,
+				CONNMAN_PROFILE_INTERFACE, "OfflineMode",
+				DBUS_TYPE_BOOLEAN, &profile->offlinemode);
 }
 
 connman_bool_t __connman_profile_get_offlinemode(void)
@@ -203,62 +134,29 @@ const char *__connman_profile_active_path(void)
 	return default_profile->path;
 }
 
-static void append_services(struct connman_profile *profile,
-						DBusMessageIter *entry)
-{
-	DBusMessageIter value, iter;
-	const char *key = "Services";
-
-	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
-		DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_OBJECT_PATH_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-
-	if (g_strcmp0(profile->ident, PROFILE_DEFAULT_IDENT) == 0)
-		__connman_service_list(&iter);
-
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(entry, &value);
-}
-
 static guint changed_timeout = 0;
 
 static gboolean services_changed(gpointer user_data)
 {
 	struct connman_profile *profile = default_profile;
-	DBusMessage *signal;
-	DBusMessageIter entry;
+	connman_dbus_append_cb_t function = NULL;
 
 	changed_timeout = 0;
 
 	if (profile == NULL)
 		return FALSE;
 
-	signal = dbus_message_new_signal(profile->path,
-				CONNMAN_PROFILE_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return FALSE;
+	if (g_strcmp0(profile->ident, PROFILE_DEFAULT_IDENT) == 0) {
+		function = __connman_service_list;
 
-	dbus_message_iter_init_append(signal, &entry);
-	append_services(profile, &entry);
-	g_dbus_send_message(connection, signal);
+		connman_dbus_property_changed_array(CONNMAN_MANAGER_PATH,
+				CONNMAN_MANAGER_INTERFACE, "Services",
+				DBUS_TYPE_OBJECT_PATH, function, NULL);
+	}
 
-	if (g_strcmp0(profile->ident, PROFILE_DEFAULT_IDENT) != 0)
-		return FALSE;
-
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return FALSE;
-
-	dbus_message_iter_init_append(signal, &entry);
-	append_services(profile, &entry);
-	g_dbus_send_message(connection, signal);
+	connman_dbus_property_changed_array(profile->path,
+				CONNMAN_PROFILE_INTERFACE, "Services",
+				DBUS_TYPE_OBJECT_PATH, function, NULL);
 
 	return FALSE;
 }
@@ -285,28 +183,6 @@ void __connman_profile_changed(gboolean delayed)
 	changed_timeout = g_timeout_add_seconds(1, services_changed, NULL);
 }
 
-int __connman_profile_add_device(struct connman_device *device)
-{
-	struct connman_service *service;
-
-	DBG("device %p", device);
-
-	service = __connman_service_create_from_device(device);
-	if (service == NULL)
-		return -EINVAL;
-
-	return 0;
-}
-
-int __connman_profile_remove_device(struct connman_device *device)
-{
-	DBG("device %p", device);
-
-	__connman_service_remove_from_device(device);
-
-	return 0;
-}
-
 int __connman_profile_add_network(struct connman_network *network)
 {
 	struct connman_service *service;
@@ -343,7 +219,7 @@ static DBusMessage *get_properties(DBusConnection *conn,
 {
 	struct connman_profile *profile = data;
 	DBusMessage *reply;
-	DBusMessageIter array, dict, entry;
+	DBusMessageIter array, dict;
 
 	DBG("conn %p", conn);
 
@@ -353,24 +229,19 @@ static DBusMessage *get_properties(DBusConnection *conn,
 
 	dbus_message_iter_init_append(reply, &array);
 
-	dbus_message_iter_open_container(&array, DBUS_TYPE_ARRAY,
-			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
-			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
-			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
+	connman_dbus_dict_open(&array, &dict);
 
 	if (profile->name != NULL)
-		connman_dbus_dict_append_variant(&dict, "Name",
+		connman_dbus_dict_append_basic(&dict, "Name",
 					DBUS_TYPE_STRING, &profile->name);
 
-	connman_dbus_dict_append_variant(&dict, "OfflineMode",
+	connman_dbus_dict_append_basic(&dict, "OfflineMode",
 				DBUS_TYPE_BOOLEAN, &profile->offlinemode);
 
-	dbus_message_iter_open_container(&dict, DBUS_TYPE_DICT_ENTRY,
-								NULL, &entry);
-	append_services(profile, &entry);
-	dbus_message_iter_close_container(&dict, &entry);
+	connman_dbus_dict_append_array(&dict, "Services",
+			DBUS_TYPE_OBJECT_PATH, __connman_service_list, NULL);
 
-	dbus_message_iter_close_container(&array, &dict);
+	connman_dbus_dict_close(&array, &dict);
 
 	return reply;
 }
@@ -449,6 +320,7 @@ static void free_profile(struct connman_profile *profile)
 {
 	g_free(profile->name);
 	g_free(profile->path);
+	g_free(profile->ident);
 	g_free(profile);
 }
 
@@ -488,7 +360,7 @@ static int create_profile(const char *ident, const char *name,
 		return -ENOMEM;
 	}
 
-	if (g_hash_table_lookup(profiles, profile->path) != NULL) {
+	if (g_hash_table_lookup(profile_hash, profile->path) != NULL) {
 		free_profile(profile);
 		return -EEXIST;
 	}
@@ -497,7 +369,7 @@ static int create_profile(const char *ident, const char *name,
 
 	__connman_storage_load_profile(profile);
 
-	g_hash_table_insert(profiles, g_strdup(profile->path), profile);
+	g_hash_table_insert(profile_hash, g_strdup(profile->path), profile);
 
 	connman_info("Adding profile %s", ident);
 
@@ -517,23 +389,6 @@ static int create_profile(const char *ident, const char *name,
 	return 0;
 }
 
-static gboolean validate_ident(const char *ident)
-{
-	unsigned int i;
-
-	for (i = 0; i < strlen(ident); i++) {
-		if (ident[i] >= '0' && ident[i] <= '9')
-			continue;
-		if (ident[i] >= 'a' && ident[i] <= 'z')
-			continue;
-		if (ident[i] >= 'A' && ident[i] <= 'Z')
-			continue;
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 int __connman_profile_create(const char *name, const char **path)
 {
 	struct connman_profile *profile;
@@ -541,14 +396,14 @@ int __connman_profile_create(const char *name, const char **path)
 
 	DBG("name %s", name);
 
-	if (validate_ident(name) == FALSE)
+	if (connman_dbus_validate_ident(name) == FALSE)
 		return -EINVAL;
 
 	err = create_profile(name, NULL, path);
 	if (err < 0)
 		return err;
 
-	profile = g_hash_table_lookup(profiles, *path);
+	profile = g_hash_table_lookup(profile_hash, *path);
 	if (profile == NULL)
 		return -EIO;
 
@@ -569,13 +424,13 @@ int __connman_profile_remove(const char *path)
 				g_strcmp0(path, default_profile->path) == 0)
 		return -EINVAL;
 
-	profile = g_hash_table_lookup(profiles, path);
+	profile = g_hash_table_lookup(profile_hash, path);
 	if (profile == NULL)
 		return -ENXIO;
 
-	__connman_storage_delete(profile->ident);
+	__connman_storage_delete_profile(profile->ident);
 
-	g_hash_table_remove(profiles, path);
+	g_hash_table_remove(profile_hash, path);
 
 	profiles_changed();
 
@@ -608,7 +463,7 @@ static int profile_init(void)
 
 			ident = g_string_free(str, FALSE);
 
-			if (validate_ident(ident) == TRUE)
+			if (connman_dbus_validate_ident(ident) == TRUE)
 				create_profile(ident, NULL, NULL);
 
 			g_free(ident);
@@ -617,7 +472,7 @@ static int profile_init(void)
 		g_dir_close(dir);
 	}
 
-	if (g_hash_table_size(profiles) == 0)
+	if (g_hash_table_size(profile_hash) == 0)
 		create_profile(PROFILE_DEFAULT_IDENT, "Default", NULL);
 
 	profiles_changed();
@@ -634,7 +489,7 @@ static int profile_load(struct connman_profile *profile)
 
 	DBG("profile %p", profile);
 
-	keyfile = __connman_storage_open(profile->ident);
+	keyfile = __connman_storage_open_profile(profile->ident);
 	if (keyfile == NULL)
 		return -EIO;
 
@@ -650,7 +505,7 @@ static int profile_load(struct connman_profile *profile)
 		profile->offlinemode = offlinemode;
 	g_clear_error(&error);
 
-	__connman_storage_close(profile->ident, keyfile, FALSE);
+	__connman_storage_close_profile(profile->ident, keyfile, FALSE);
 
 	return 0;
 }
@@ -661,7 +516,7 @@ static int profile_save(struct connman_profile *profile)
 
 	DBG("profile %p", profile);
 
-	keyfile = __connman_storage_open(profile->ident);
+	keyfile = __connman_storage_open_profile(profile->ident);
 	if (keyfile == NULL)
 		return -EIO;
 
@@ -672,7 +527,7 @@ static int profile_save(struct connman_profile *profile)
 	g_key_file_set_boolean(keyfile, "global",
 					"OfflineMode", profile->offlinemode);
 
-	__connman_storage_close(profile->ident, keyfile, TRUE);
+	__connman_storage_close_profile(profile->ident, keyfile, TRUE);
 
 	return 0;
 }
@@ -685,18 +540,18 @@ static struct connman_storage profile_storage = {
 	.profile_save	= profile_save,
 };
 
-int __connman_profile_init(DBusConnection *conn)
+int __connman_profile_init(void)
 {
-	DBG("conn %p", conn);
+	DBG("");
 
-	connection = dbus_connection_ref(conn);
+	connection = connman_dbus_get_connection();
 	if (connection == NULL)
 		return -1;
 
 	if (connman_storage_register(&profile_storage) < 0)
 		connman_error("Failed to register profile storage");
 
-	profiles = g_hash_table_new_full(g_str_hash, g_str_equal,
+	profile_hash = g_hash_table_new_full(g_str_hash, g_str_equal,
 						g_free, unregister_profile);
 
 	return 0;
@@ -704,17 +559,15 @@ int __connman_profile_init(DBusConnection *conn)
 
 void __connman_profile_cleanup(void)
 {
-	DBG("conn %p", connection);
-
-	if (profiles != NULL) {
-		g_hash_table_destroy(profiles);
-		profiles = NULL;
-	}
-
-	connman_storage_unregister(&profile_storage);
+	DBG("");
 
 	if (connection == NULL)
 		return;
 
+	g_hash_table_destroy(profile_hash);
+	profile_hash = NULL;
+
+	connman_storage_unregister(&profile_storage);
+
 	dbus_connection_unref(connection);
 }
